@@ -106,7 +106,9 @@ on_context_available (GUPnPContextManager *context_manager,
 static gboolean
 init_upnp (void)
 {
+#if !GLIB_CHECK_VERSION(2, 35, 0)
         g_type_init ();
+#endif
 
         context_manager = gupnp_context_manager_new (NULL, 0);
         g_assert (context_manager != NULL);
@@ -139,9 +141,6 @@ gint
 main (gint   argc,
       gchar *argv[])
 {
-        /* First thing initialze the thread system */
-        g_thread_init (NULL);
-
         if (!init_ui (&argc, &argv)) {
            return -2;
         }
