@@ -54,6 +54,7 @@ static GtkWidget *popup;
 static GtkWidget *didl_dialog;
 static GtkWidget *didl_textview;
 static gboolean   expanded;
+static GHashTable *initial_notify;
 
 typedef struct
 {
@@ -341,10 +342,16 @@ setup_playlist_treeview (GtkBuilder *builder)
         GtkTreeModel      *model;
         GtkTreeSelection  *selection;
 
+        initial_notify = g_hash_table_new (g_direct_hash, g_direct_equal);
+
         treeview = GTK_WIDGET (gtk_builder_get_object (builder,
                                                        "playlist-treeview"));
         g_assert (treeview != NULL);
 
+        g_object_weak_ref (G_OBJECT (treeview),
+                           (GWeakNotify) g_hash_table_destroy,
+                           initial_notify);
+
         popup = GTK_WIDGET (gtk_builder_get_object (builder, "playlist-popup"));
         g_assert (popup != NULL);
 
@@ -616,6 +623,15 @@ on_container_update_ids (GUPnPServiceProxy *content_dir,
         char **tokens;
         guint  i;
 
+        /* Ignore initial event. It might have some old updates that happened
+         * ages ago and cause duplicate entries in our list.
+         */
+        if (g_hash_table_contains (initial_notify, content_dir)) {
+                g_hash_table_remove (initial_notify, content_dir);
+
+                return;
+        }
+
         tokens = g_strsplit (g_value_get_string (value), ",", 0);
         for (i = 0;
              tokens[i] != NULL && tokens[i+1] != NULL;
@@ -790,7 +806,8 @@ browse_cb (GUPnPServiceProxy       *content_dir,
                                         NULL);
         if (didl_xml) {
                 GUPnPDIDLLiteParser *parser;
-                guint32              remaining;
+                gint32              remaining;
+                gint32              batch_size;
                 GError              *error;
 
                 error = NULL;
@@ -801,12 +818,17 @@ browse_cb (GUPnPServiceProxy       *content_dir,
                                   G_CALLBACK (on_didl_object_available),
                                   data);
 
-                if (!gupnp_didl_lite_parser_parse_didl (parser,
-                                                        didl_xml,
-                                                        &error)) {
-                        g_warning ("%s", error->message);
-                        g_error_free (error);
-                }
+                /* Only try to parse DIDL if server claims that there was a
+                 * result */
+                if (number_returned > 0)
+                        if (!gupnp_didl_lite_parser_parse_didl (parser,
+                                                                didl_xml,
+                                                                &error)) {
+                                g_warning ("Error while browsing %s: %s",
+                                           data->id,
+                                           error->message);
+                                g_error_free (error);
+                        }
 
                 g_object_unref (parser);
                 g_free (didl_xml);
@@ -815,13 +837,19 @@ browse_cb (GUPnPServiceProxy       *content_dir,
 
                 /* See if we have more objects to get */
                 remaining = total_matches - data->starting_index;
+
                 /* Keep browsing till we get each and every object */
-                if (remaining != 0)
+                if ((remaining > 0 || total_matches == 0) && number_returned != 0) {
+                        if (remaining > 0)
+                                batch_size = MIN (remaining, MAX_BROWSE);
+                        else
+                                batch_size = MAX_BROWSE;
+
                         browse (content_dir,
                                 data->id,
                                 data->starting_index,
-                                MIN (remaining, MAX_BROWSE));
-                else
+                                batch_size);
+                } else
                         update_container_child_count (content_dir, data->id);
         } else if (error) {
                 GUPnPServiceInfo *info;
@@ -999,6 +1027,7 @@ append_media_server (GUPnPDeviceProxy *proxy,
                                                 on_container_update_ids,
                                                 NULL);
                 gupnp_service_proxy_set_subscribed (content_dir, TRUE);
+                g_hash_table_insert (initial_notify, content_dir, content_dir);
 
                 g_object_unref (content_dir);
         }
